package require Tcl 8.5

proc listConcat {a b c} {
   list $a $b {*}$c
}

proc Fibonacci {n} {
    set a 0
    set b 1
    for {set i 2} {$i <= $n} {incr i} {
        set b [expr {$a + [set a $b]}]
    }
    return $b
}

proc iter {param} {
    set result {}
    foreach x $param {
        lappend result [string length $param]
    }
    return $result
}

proc inner {} {
    return "ok"
}
proc outer {} {
    inner; inner; inner; inner; inner
}

proc loopcb {x} {
    for {set i 0} {$i < 10000} {incr i} {
        if {$i == $x} break
        continue
    }
    return "ok"
}

proc ensDispatch1 {} {
    info tclversion
    info patchlevel
}

proc ensDispatch2 {} {
    namespace current
    info level
}

proc ensDispatch3 {} {
    namespace inscope :: {return -level 0 "ok"}
    namespace inscope :: {return -level 0 "ok"}
}

proc ensDispatch4 {} {
    ens4 foo bar
}
namespace ensemble create -command ens4 -map {
    foo {::ens4core}
}
proc ens4core {msg} {
    return $msg
}

proc dictWithAdd {d} {
    dict with d {}
    return [expr {$a + $b}]
}

proc tryNormal {} {
    set d 1.875
    try {
        set x [expr {$d / $d}]
    } on error {} {
        set x "error happened"
    }
    return $x
}

proc tryError {} {
    set d 0.0
    try {
        set x [expr {$d / $d}]
    } on error {} {
        set x "error happened"
    }
    return $x
}

proc tryNested {} {
    set d 0.0
    catch {
        try {
            set x [expr {$d / $d}]
        } on error {} {
            error "error happened"
        }
    } msg opt
    return $opt
}

proc tryNestedOver {} {
    set d 0.0
    catch {
        try {
            set x [expr {$d / $d}]
        } on error {} {
            set x "error happened"
        }
    } msg opt
    return $opt
}

proc benchmark {title script {version 8.5}} {
    if {[package vsatisfies [info patchlevel] $version]} {
       eval $script
       for {set i 0} {$i < 20} {incr i} {
            lappend t [lindex [time $script 100000] 0]
       }
       puts [format "%s: %4f" $title [tcl::mathfunc::min {*}$t]]
    } else {
        puts [format "%s: N/A" $title]
    }
}

benchmark "ListConcat"    { listConcat {a b c} {d e f} {g h i} }
benchmark "Fibonacci"     { Fibonacci 10 }
benchmark "ListIterate"   { iter {a aaa aaaaa} }
benchmark "ProcCall"      { outer }
benchmark "LoopCB"        { loopcb 10}
benchmark "EnsDispatch1"  { ensDispatch1 }
benchmark "EnsDispatch2"  { ensDispatch2 }
benchmark "EnsDispatch3"  { ensDispatch3 }
benchmark "EnsDispatch4"  { ensDispatch4 }
benchmark "DictWith"      { dictWithAdd {a 1 b 2 c 4} }
benchmark "TryNormal"     { tryNormal }      8.6
benchmark "TryError"      { tryError }       8.6
benchmark "TryNested"     { tryNested }      8.6
benchmark "TryNestedOver" { tryNestedOver }  8.6
